# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: view.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nview.proto\".\n\x0b\x44mSegConfig\x12\x10\n\x08pageSize\x18\x01 \x01(\x03\x12\r\n\x05total\x18\x02 \x01(\x03\"H\n\x11\x44\x61nmakuFlagConfig\x12\x0f\n\x07recFlag\x18\x01 \x01(\x05\x12\x0f\n\x07recText\x18\x02 \x01(\t\x12\x11\n\trecSwitch\x18\x03 \x01(\x05\"\xa1\x01\n\tCommandDm\x12\n\n\x02id\x18\x01 \x01(\x03\x12\x0b\n\x03oid\x18\x02 \x01(\x03\x12\x0b\n\x03mid\x18\x03 \x01(\x03\x12\x0f\n\x07\x63ommand\x18\x04 \x01(\t\x12\x0f\n\x07\x63ontent\x18\x05 \x01(\t\x12\x10\n\x08progress\x18\x06 \x01(\x05\x12\r\n\x05\x63time\x18\x07 \x01(\t\x12\r\n\x05mtime\x18\x08 \x01(\t\x12\r\n\x05\x65xtra\x18\t \x01(\t\x12\r\n\x05idStr\x18\n \x01(\t\"\x8b\x03\n\x14\x44\x61nmuWebPlayerConfig\x12\x10\n\x08\x64mSwitch\x18\x01 \x01(\x08\x12\x10\n\x08\x61iSwitch\x18\x02 \x01(\x08\x12\x0f\n\x07\x61iLevel\x18\x03 \x01(\x05\x12\x10\n\x08\x62locktop\x18\x04 \x01(\x08\x12\x13\n\x0b\x62lockscroll\x18\x05 \x01(\x08\x12\x13\n\x0b\x62lockbottom\x18\x06 \x01(\x08\x12\x12\n\nblockcolor\x18\x07 \x01(\x08\x12\x14\n\x0c\x62lockspecial\x18\x08 \x01(\x08\x12\x14\n\x0cpreventshade\x18\t \x01(\x08\x12\r\n\x05\x64mask\x18\n \x01(\x08\x12\x0f\n\x07opacity\x18\x0b \x01(\x02\x12\x0e\n\x06\x64marea\x18\x0c \x01(\x05\x12\x11\n\tspeedplus\x18\r \x01(\x02\x12\x10\n\x08\x66ontsize\x18\x0e \x01(\x02\x12\x12\n\nscreensync\x18\x0f \x01(\x08\x12\x11\n\tspeedsync\x18\x10 \x01(\x08\x12\x12\n\nfontfamily\x18\x11 \x01(\t\x12\x0c\n\x04\x62old\x18\x12 \x01(\x08\x12\x12\n\nfontborder\x18\x13 \x01(\x05\x12\x10\n\x08\x64rawType\x18\x14 \x01(\t\"\xfd\x01\n\x0e\x44mWebViewReply\x12\r\n\x05state\x18\x01 \x01(\x05\x12\x0c\n\x04text\x18\x02 \x01(\t\x12\x10\n\x08textSide\x18\x03 \x01(\t\x12\x1b\n\x05\x64mSge\x18\x04 \x01(\x0b\x32\x0c.DmSegConfig\x12 \n\x04\x66lag\x18\x05 \x01(\x0b\x32\x12.DanmakuFlagConfig\x12\x12\n\nspecialDms\x18\x06 \x03(\t\x12\x10\n\x08\x63heckBox\x18\x07 \x01(\x08\x12\r\n\x05\x63ount\x18\x08 \x01(\x03\x12\x1e\n\ncommandDms\x18\t \x03(\x0b\x32\n.CommandDm\x12(\n\tdmSetting\x18\n \x01(\x0b\x32\x15.DanmuWebPlayerConfigb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'view_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _DMSEGCONFIG._serialized_start=14
  _DMSEGCONFIG._serialized_end=60
  _DANMAKUFLAGCONFIG._serialized_start=62
  _DANMAKUFLAGCONFIG._serialized_end=134
  _COMMANDDM._serialized_start=137
  _COMMANDDM._serialized_end=298
  _DANMUWEBPLAYERCONFIG._serialized_start=301
  _DANMUWEBPLAYERCONFIG._serialized_end=696
  _DMWEBVIEWREPLY._serialized_start=699
  _DMWEBVIEWREPLY._serialized_end=952
# @@protoc_insertion_point(module_scope)
